{% extends "admin/change_form.html" %} {% load i18n static admin_list admin_urls %} {% block object-tools-items %}
<li>
  <a href="{% url 'admin:campaign_drip_timeline' original.id 4 7 %}" class=""
    >View Timeline</a
  >
</li>
<li>
  <a href="{% url 'admin:drip_campaign_history' original.pk %}" class="historylink">{% translate "History" %}</a>
</li>
{% if has_absolute_url %}
<li>
  <a
    href="../../../r/{{ content_type_id }}/{{ object_id }}/"
    class="viewsitelink"
    >{% trans "View on site" %}</a
  >
</li>
{% endif%}
{% endblock %}
